@@ -1,14 +1,17 @@
-from __future__ import absolute_import
-
 from pkgutil import extend_path
 __path__ = extend_path(__path__, __name__)
 
 from .pubmedarticle import PubMedArticle
+from .pubmedauthor import PubMedAuthor
 from .pubmedfetcher import PubMedFetcher
 from .medgenfetcher import MedGenFetcher
 from .medgenconcept import MedGenConcept
-from .crossref import CrossRef
+from .clinvarfetcher import ClinVarFetcher
+from .clinvarvariant import ClinVarVariant
+from .crossref import CrossRefFetcher
 from .findit import FindIt
+from .dx_doi import DxDOI
+from .urlreverse import UrlReverse
 
-__version__ = '0.3.13.1'
+__version__ = '0.5'
 
